@@ -25,6 +25,7 @@
 #include "../include/Dialog_Partition_Info.h"
 #include "../include/DialogManageFlags.h"
 #include "../include/OperationCopy.h"
+#include "../include/OperationCheck.h"
 #include "../include/OperationCreate.h"
 #include "../include/OperationDelete.h"
 #include "../include/OperationFormat.h"
@@ -48,6 +49,19 @@ Win_GParted::Win_GParted( const std::vector<Glib::ustring> & user_devices )
 	OPERATIONSLIST_OPEN = true ;
 	gparted_core .set_user_devices( user_devices ) ;
 	
+	MENU_NEW = TOOLBAR_NEW =
+        MENU_DEL = TOOLBAR_DEL =
+        MENU_RESIZE_MOVE = TOOLBAR_RESIZE_MOVE =
+        MENU_COPY = TOOLBAR_COPY =
+        MENU_PASTE = TOOLBAR_PASTE =
+        MENU_FORMAT =
+        MENU_TOGGLE_MOUNT_SWAP =
+        MENU_MOUNT =
+        MENU_FLAGS =
+        MENU_INFO =
+        TOOLBAR_UNDO =
+        TOOLBAR_APPLY = -1 ;
+
 	//==== GUI =========================
 	this ->set_title( _("GParted") );
 	this ->set_default_size( 775, 500 );
@@ -92,9 +106,12 @@ Win_GParted::Win_GParted( const std::vector<Glib::ustring> & user_devices )
 	init_device_info() ;
 	
 	//operationslist...
-	init_operationslist() ;
+	hbox_operations .signal_undo .connect( sigc::mem_fun( this, &Win_GParted::activate_undo ) ) ;
+	hbox_operations .signal_clear .connect( sigc::mem_fun( this, &Win_GParted::clear_operationslist ) ) ;
+	hbox_operations .signal_apply .connect( sigc::mem_fun( this, &Win_GParted::activate_apply ) ) ;
+	hbox_operations .signal_close .connect( sigc::mem_fun( this, &Win_GParted::close_operationslist ) ) ;
 	vpaned_main .pack2( hbox_operations, true, true ) ;
-	
+
 	//statusbar... 
 	pulsebar .set_pulse_step( 0.01 );
 	statusbar .add( pulsebar );
@@ -112,32 +129,54 @@ void Win_GParted::init_menubar()
 	//gparted
 	menu = manage( new Gtk::Menu() ) ;
 	image = manage( new Gtk::Image( Gtk::Stock::REFRESH, Gtk::ICON_SIZE_MENU ) );
-	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( _("_Refresh devices"), Gtk::AccelKey("<control>r"), *image, sigc::mem_fun(*this, &Win_GParted::menu_gparted_refresh_devices) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem(
+		_("_Refresh Devices"),
+		Gtk::AccelKey("<control>r"),
+		*image, 
+		sigc::mem_fun(*this, &Win_GParted::menu_gparted_refresh_devices) ) );
 	
 	image = manage( new Gtk::Image( Gtk::Stock::HARDDISK, Gtk::ICON_SIZE_MENU ) );
-	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( _("Devices"), *image ) ) ; 
+	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( _("_Devices"), *image ) ) ; 
 	
 	menu ->items() .push_back( Gtk::Menu_Helpers::SeparatorElem( ) );
-	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem( _("Features"), sigc::mem_fun( *this, &Win_GParted::menu_gparted_features ) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem(
+		_("_Show Features"), sigc::mem_fun( *this, &Win_GParted::menu_gparted_features ) ) );
 	menu ->items() .push_back( Gtk::Menu_Helpers::SeparatorElem( ) );
-	menu ->items() .push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::QUIT, sigc::mem_fun(*this, &Win_GParted::menu_gparted_quit) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::StockMenuElem( 
+		Gtk::Stock::QUIT, sigc::mem_fun(*this, &Win_GParted::menu_gparted_quit) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_GParted"), *menu ) );
 	
 	//edit
 	menu = manage( new Gtk::Menu() ) ;
-	menu ->items() .push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::UNDO, Gtk::AccelKey("<control>z"), sigc::mem_fun(*this, &Win_GParted::activate_undo) ) );
-	menu ->items() .push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::APPLY, sigc::mem_fun(*this, &Win_GParted::activate_apply) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( 
+		_("_Undo Last Operation"), 
+		Gtk::AccelKey("<control>z"),
+		* manage( new Gtk::Image( Gtk::Stock::UNDO, Gtk::ICON_SIZE_MENU ) ), 
+		sigc::mem_fun(*this, &Win_GParted::activate_undo) ) );
+
+	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( 
+		_("_Clear All Operations"), 
+		* manage( new Gtk::Image( Gtk::Stock::CLEAR, Gtk::ICON_SIZE_MENU ) ), 
+		sigc::mem_fun(*this, &Win_GParted::clear_operationslist) ) );
+
+	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( 
+		_("_Apply All Operations"), 
+		* manage( new Gtk::Image( Gtk::Stock::APPLY, Gtk::ICON_SIZE_MENU ) ), 
+		sigc::mem_fun(*this, &Win_GParted::activate_apply) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_Edit"), *menu ) );
 	
 	//view
 	menu = manage( new Gtk::Menu() ) ;
-	menu ->items() .push_back( Gtk::Menu_Helpers::CheckMenuElem( _("Harddisk Information"), sigc::mem_fun(*this, &Win_GParted::menu_view_harddisk_info) ) );
-	menu ->items() .push_back( Gtk::Menu_Helpers::CheckMenuElem( _("Operations"), sigc::mem_fun(*this, &Win_GParted::menu_view_operations) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::CheckMenuElem(
+		_("Device _Information"), sigc::mem_fun(*this, &Win_GParted::menu_view_harddisk_info) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::CheckMenuElem( 
+		_("Pending _Operations"), sigc::mem_fun(*this, &Win_GParted::menu_view_operations) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_View"), *menu ) );
 	
 	//device
 	menu = manage( new Gtk::Menu() ) ;
-	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem( _("Set Disklabel"), sigc::mem_fun(*this, &Win_GParted::activate_disklabel) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem( Glib::ustring( _("_Set Disklabel") ) + " ...",
+								sigc::mem_fun(*this, &Win_GParted::activate_disklabel) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_Device"), *menu ) );
 		
 	//partition
@@ -146,59 +185,72 @@ void Win_GParted::init_menubar()
 	
 	//help
 	menu = manage( new Gtk::Menu() ) ;
-	menu ->items() .push_back(Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::HELP, sigc::mem_fun(*this, &Win_GParted::menu_help_contents) ) );
-	menu ->items( ) .push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::ABOUT, sigc::mem_fun(*this, &Win_GParted::menu_help_about) ) );
+	menu ->items() .push_back(Gtk::Menu_Helpers::StockMenuElem(
+		Gtk::Stock::HELP, sigc::mem_fun(*this, &Win_GParted::menu_help_contents) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::StockMenuElem(
+		Gtk::Stock::ABOUT, sigc::mem_fun(*this, &Win_GParted::menu_help_about) ) );
 
 	menubar_main.items() .push_back( Gtk::Menu_Helpers::MenuElem(_("_Help"), *menu ) );
 }
 
 void Win_GParted::init_toolbar() 
 {
+	int index = 0 ;
 	//initialize and pack toolbar_main 
 	hbox_toolbar.pack_start( toolbar_main );
 	
 	//NEW and DELETE
-	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::NEW));
-	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_new) );
-	toolbar_main.append(*toolbutton);
+	toolbutton = Gtk::manage( new Gtk::ToolButton( Gtk::Stock::NEW ) );
+	toolbutton ->signal_clicked() .connect( sigc::mem_fun( *this, &Win_GParted::activate_new ) );
+	toolbar_main .append( *toolbutton );
+	TOOLBAR_NEW = index++ ;
 	toolbutton ->set_tooltip(tooltips, _("Create a new partition in the selected unallocated space") );		
 	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::DELETE));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_delete) );
 	toolbar_main.append(*toolbutton);
+	TOOLBAR_DEL = index++ ;
 	toolbutton ->set_tooltip(tooltips, _("Delete the selected partition") );		
 	toolbar_main.append( *(Gtk::manage(new Gtk::SeparatorToolItem)) );
+	index++ ;
 	
 	//RESIZE/MOVE
 	image = manage( new Gtk::Image( Gtk::Stock::GOTO_LAST, Gtk::ICON_SIZE_BUTTON ) );
 	toolbutton = Gtk::manage(new Gtk::ToolButton( *image, _("Resize/Move") ));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_resize) );
 	toolbar_main.append(*toolbutton);
+	TOOLBAR_RESIZE_MOVE = index++ ;
 	toolbutton ->set_tooltip(tooltips, _("Resize/Move the selected partition") );		
 	toolbar_main.append( *(Gtk::manage(new Gtk::SeparatorToolItem)) );
-	
+	index++ ;
+
 	//COPY and PASTE
 	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::COPY));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_copy) );
 	toolbar_main.append(*toolbutton);
+	TOOLBAR_COPY = index++ ;
 	toolbutton ->set_tooltip(tooltips, _("Copy the selected partition to the clipboard") );		
 	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::PASTE));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_paste) );
 	toolbar_main.append(*toolbutton);
+	TOOLBAR_PASTE = index++ ;
 	toolbutton ->set_tooltip(tooltips, _("Paste the partition from the clipboard") );		
 	toolbar_main.append( *(Gtk::manage(new Gtk::SeparatorToolItem)) );
+	index++ ;
 	
 	//UNDO and APPLY
 	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::UNDO));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_undo) );
 	toolbar_main.append(*toolbutton);
+	TOOLBAR_UNDO = index++ ;
 	toolbutton ->set_sensitive( false );
-	toolbutton ->set_tooltip(tooltips, _("Undo last operation") );		
+	toolbutton ->set_tooltip(tooltips, _("Undo Last Operation") );		
 	
 	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::APPLY));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_apply) );
 	toolbar_main.append(*toolbutton);
+	TOOLBAR_APPLY = index++ ;
 	toolbutton ->set_sensitive( false );
-	toolbutton ->set_tooltip(tooltips, _("Apply all operations") );		
+	toolbutton ->set_tooltip(tooltips, _("Apply All Operations") );		
 	
 	//initialize and pack combo_devices
 	liststore_devices = Gtk::ListStore::create( treeview_devices_columns ) ;
@@ -215,34 +267,44 @@ void Win_GParted::init_toolbar()
 
 void Win_GParted::init_partition_menu() 
 {
+	int index = 0 ;
 	//fill menu_partition
 	menu_partition .items() .push_back( 
 			Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::NEW,
 							  sigc::mem_fun(*this, &Win_GParted::activate_new) ) );
+	MENU_NEW = index++ ;
+	
 	menu_partition .items() .push_back( 
 			Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::DELETE, 
 							  Gtk::AccelKey( GDK_Delete, Gdk::BUTTON1_MASK ),
 							  sigc::mem_fun(*this, &Win_GParted::activate_delete) ) );
+	MENU_DEL = index++ ;
 
 	menu_partition .items() .push_back( Gtk::Menu_Helpers::SeparatorElem() );
+	index++ ;
 	
 	image = manage( new Gtk::Image( Gtk::Stock::GOTO_LAST, Gtk::ICON_SIZE_MENU ) );
 	menu_partition .items() .push_back( 
 			Gtk::Menu_Helpers::ImageMenuElem( _("_Resize/Move"), 
 							  *image, 
 							  sigc::mem_fun(*this, &Win_GParted::activate_resize) ) );
+	MENU_RESIZE_MOVE = index++ ;
 	
 	menu_partition .items() .push_back( Gtk::Menu_Helpers::SeparatorElem() );
+	index++ ;
 	
 	menu_partition .items() .push_back( 
 			Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::COPY,
 							  sigc::mem_fun(*this, &Win_GParted::activate_copy) ) );
+	MENU_COPY = index++ ;
 	
 	menu_partition .items() .push_back( 
 			Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::PASTE,
 							  sigc::mem_fun(*this, &Win_GParted::activate_paste) ) );
+	MENU_PASTE = index++ ;
 	
 	menu_partition .items() .push_back( Gtk::Menu_Helpers::SeparatorElem() );
+	index++ ;
 	
 	image = manage( new Gtk::Image( Gtk::Stock::CONVERT, Gtk::ICON_SIZE_MENU ) );
 	/*TO TRANSLATORS: menuitem which holds a submenu with filesystems.. */
@@ -250,28 +312,41 @@ void Win_GParted::init_partition_menu()
 			Gtk::Menu_Helpers::ImageMenuElem( _("_Format to"),
 							  *image,
 							  * create_format_menu() ) ) ;
+	MENU_FORMAT = index++ ;
 	
-	menu_partition .items() .push_back( Gtk::Menu_Helpers::SeparatorElem() );
+	menu_partition .items() .push_back( Gtk::Menu_Helpers::SeparatorElem() ) ;
+	index++ ;
 	
 	menu_partition .items() .push_back(
-			Gtk::Menu_Helpers::MenuElem( _("unmount"),
+			Gtk::Menu_Helpers::MenuElem( _("Unmount"),
 						     sigc::mem_fun( *this, &Win_GParted::toggle_swap_mount_state ) ) );
+	MENU_TOGGLE_MOUNT_SWAP = index++ ;
 		
 	/*TO TRANSLATORS: menuitem which holds a submenu with mountpoints.. */
 	menu_partition .items() .push_back(
-			Gtk::Menu_Helpers::MenuElem( _("mount on"), * manage( new Gtk::Menu() ) ) ) ;
+			Gtk::Menu_Helpers::MenuElem( _("_Mount on"), * manage( new Gtk::Menu() ) ) ) ;
+	MENU_MOUNT = index++ ;
 
 	menu_partition .items() .push_back( Gtk::Menu_Helpers::SeparatorElem() ) ;
+	index++ ;
 
 	menu_partition .items() .push_back(
-			Gtk::Menu_Helpers::MenuElem( _("manage flags"),
+			Gtk::Menu_Helpers::MenuElem( _("M_anage Flags"),
 						     sigc::mem_fun( *this, &Win_GParted::activate_manage_flags ) ) );
+	MENU_FLAGS = index++ ;
+
+	menu_partition .items() .push_back(
+			Gtk::Menu_Helpers::MenuElem( _("C_heck"),
+						     sigc::mem_fun( *this, &Win_GParted::activate_check ) ) );
+	MENU_CHECK = index++ ;
 
 	menu_partition .items() .push_back( Gtk::Menu_Helpers::SeparatorElem() ) ;
+	index++ ;
 	
 	menu_partition .items() .push_back( 
 			Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::DIALOG_INFO,
 							  sigc::mem_fun(*this, &Win_GParted::activate_info) ) );
+	MENU_INFO = index++ ;
 	
 	menu_partition .accelerate( *this ) ;  
 }
@@ -315,7 +390,7 @@ void Win_GParted::init_device_info()
 	
 	//title
 	vbox_info .pack_start( 
-		* Utils::mk_label( " <b>" + static_cast<Glib::ustring>( _("Harddisk Information") ) + ":</b>" ),
+		* Utils::mk_label( " <b>" + static_cast<Glib::ustring>( _("Device Information") ) + ":</b>" ),
 		Gtk::PACK_SHRINK );
 	
 	//GENERAL DEVICE INFO
@@ -402,59 +477,18 @@ void Win_GParted::init_device_info()
 	vbox_info .pack_start( *table, Gtk::PACK_SHRINK );
 }
 
-void Win_GParted::init_operationslist() 
-{
-	//create listview for pending operations
-	liststore_operations = Gtk::ListStore::create( treeview_operations_columns );
-	treeview_operations .set_model( liststore_operations );
-	treeview_operations .set_headers_visible( false );
-	treeview_operations .append_column( "", treeview_operations_columns .operation_icon );
-	treeview_operations .append_column( "", treeview_operations_columns .operation_description );
-	treeview_operations .get_selection() ->set_mode( Gtk::SELECTION_NONE );
-
-	//init scrollwindow_operations
-	scrollwindow = manage( new Gtk::ScrolledWindow( ) ) ;
-	scrollwindow ->set_shadow_type( Gtk::SHADOW_ETCHED_IN );
-	scrollwindow ->set_policy( Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC );
-	scrollwindow ->add ( treeview_operations ) ;
-	
-	//set up the close and clear buttons and pack them in a vbox
-	vbox = manage( new Gtk::VBox( ) ) ;
-	//CLOSE
-	button = manage( new Gtk::Button( ) );
-	image = manage( new Gtk::Image( Gtk::Stock::CLOSE, Gtk::ICON_SIZE_MENU ) );
-	button ->add( *image ) ;
-	button ->set_relief( Gtk::RELIEF_NONE );
-	tooltips .set_tip( *button, _("Hide operationslist") );
-	button ->signal_clicked( ) .connect( sigc::mem_fun( *this, &Win_GParted::close_operationslist ) );
-	vbox ->pack_start( *button, Gtk::PACK_SHRINK );
-	
-	//CLEAR
-	button = manage( new Gtk::Button( ) );
-	image = manage( new Gtk::Image( Gtk::Stock::CLEAR, Gtk::ICON_SIZE_MENU ) );
-	button ->add( *image ) ;
-	button ->set_relief( Gtk::RELIEF_NONE );
-	tooltips .set_tip( *button, _("Clear operationslist") );
-	button ->signal_clicked( ) .connect( sigc::mem_fun( *this, &Win_GParted::clear_operationslist ) );
-	vbox ->pack_start( *button, Gtk::PACK_SHRINK );
-	
-	//add vbox and scrollwindow_operations to hbox_operations
-	hbox_operations .pack_start( *vbox, Gtk::PACK_SHRINK );
-	hbox_operations .pack_start( *scrollwindow, Gtk::PACK_EXPAND_WIDGET );
-}
-
-void Win_GParted::init_hpaned_main( ) 
+void Win_GParted::init_hpaned_main() 
 {
 	//left scrollwindow (holds device info)
-	scrollwindow = manage( new Gtk::ScrolledWindow( ) ) ;
+	scrollwindow = manage( new Gtk::ScrolledWindow() ) ;
 	scrollwindow ->set_shadow_type( Gtk::SHADOW_ETCHED_IN );
 	scrollwindow ->set_policy( Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC );
 
-	hpaned_main.pack1( *scrollwindow, true, true );
+	hpaned_main .pack1( *scrollwindow, true, true );
 	scrollwindow ->add( vbox_info );
 
 	//right scrollwindow (holds treeview with partitions)
-	scrollwindow = manage( new Gtk::ScrolledWindow( ) ) ;
+	scrollwindow = manage( new Gtk::ScrolledWindow() ) ;
 	scrollwindow ->set_shadow_type( Gtk::SHADOW_ETCHED_IN );
 	scrollwindow ->set_policy( Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC );
 	
@@ -463,7 +497,7 @@ void Win_GParted::init_hpaned_main( )
 	treeview_detail .signal_partition_activated .connect( sigc::mem_fun( this, &Win_GParted::on_partition_activated ) );
 	treeview_detail .signal_popup_menu .connect( sigc::mem_fun( this, &Win_GParted::on_partition_popup_menu ) );
 	scrollwindow ->add( treeview_detail );
-	hpaned_main.pack2( *scrollwindow, true, true );
+	hpaned_main .pack2( *scrollwindow, true, true );
 }
 
 void Win_GParted::refresh_combo_devices()
@@ -573,80 +607,56 @@ bool Win_GParted::on_delete_event( GdkEventAny *event )
 	return ! Quit_Check_Operations();
 }	
 
-void Win_GParted::Add_Operation( OperationType operationtype, const Partition & new_partition, int index )
+void Win_GParted::Add_Operation( Operation * operation, int index )
 {
-	Operation * operation ;
-	switch ( operationtype )
-	{		
-		case GParted::DELETE	:
-			operation = new OperationDelete( devices[ current_device ], selected_partition ) ;
-			operation ->icon = render_icon( Gtk::Stock::DELETE, Gtk::ICON_SIZE_MENU ) ;
-			break;
-		case GParted::CREATE	: 
-			operation = new OperationCreate( devices[ current_device ],
-							 selected_partition,
-							 new_partition ) ;
-			operation ->icon = render_icon( Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU );
-			break;
-		case GParted::RESIZE_MOVE:
-			operation = new OperationResizeMove( devices[ current_device ],
-							     selected_partition,
-							     new_partition );
-			operation ->icon = render_icon( Gtk::Stock::GOTO_LAST, Gtk::ICON_SIZE_MENU );
-			break;
-		case GParted::FORMAT	:
-			operation = new OperationFormat( devices[ current_device ],
-							 selected_partition,
-							 new_partition );
-			operation ->icon = render_icon( Gtk::Stock::CONVERT, Gtk::ICON_SIZE_MENU );
-			break;
-		case GParted::COPY	:
-			operation = new OperationCopy( devices[ current_device ],
-					 	       selected_partition,
-						       new_partition,
-						       copied_partition ) ;
-			operation ->icon = render_icon( Gtk::Stock::COPY, Gtk::ICON_SIZE_MENU );
-			break;
-	}
-
 	if ( operation )
-	{
-		if ( index >= 0 && index < static_cast<int>( operations .size() ) )
-			operations .insert( operations .begin() + index, operation ) ;
-		else
-			operations .push_back( operation );
+	{ 
+		Glib::ustring error ;
+		//FIXME: this is becoming a mess.. maybe it's better to check if partition_new > 0
+		if ( operation ->type == OPERATION_DELETE || 
+		     operation ->type == OPERATION_FORMAT || 
+		     operation ->type == OPERATION_CHECK ||
+		     gparted_core .snap_to_cylinder( operation ->device, operation ->partition_new, error ) )
+		{
+			operation ->create_description() ;
 
-		allow_undo( true ) ;
-		allow_apply( true ) ;
+			if ( index >= 0 && index < static_cast<int>( operations .size() ) )
+				operations .insert( operations .begin() + index, operation ) ;
+			else
+				operations .push_back( operation );
+
+			allow_undo_clear_apply( true ) ;
+			Refresh_Visual();
+			
+			if ( operations .size() == 1 ) //first operation, open operationslist
+				open_operationslist() ;
+		}
+		else
+		{
+			Gtk::MessageDialog dialog( *this,
+				   _("Could not add this operation to the list."),
+				   false,
+				   Gtk::MESSAGE_ERROR,
+				   Gtk::BUTTONS_OK,
+				   true );
+			dialog .set_secondary_text( error ) ;
+
+			dialog .run() ;
+		}
 	}
-	
-	Refresh_Visual();
-	
-	if ( operations .size() == 1 ) //first operation, open operationslist
-		open_operationslist() ;
-	
-	//make scrollwindow focus on the last operation in the list	
-	treeview_operations .set_cursor(
-		static_cast<Gtk::TreePath>( static_cast<Gtk::TreeRow>( 
-				*(--liststore_operations ->children() .end()) ) ) ) ;
 }
 
 void Win_GParted::Refresh_Visual()
 {
 	std::vector<Partition> partitions = devices[ current_device ] .partitions ; 
-	liststore_operations ->clear();
 	
 	//make all operations visible
 	for ( unsigned int t = 0 ; t < operations .size(); t++ )
-	{	
 		if ( operations[ t ] ->device == devices[ current_device ] )
 			operations[ t ] ->apply_to_visual( partitions ) ;
 			
-		treerow = *( liststore_operations ->append() );
-		treerow[ treeview_operations_columns .operation_description ] = operations[ t ] ->description ;
-		treerow[ treeview_operations_columns .operation_icon ] = operations[ t ] ->icon ;
-	}
-	
+	hbox_operations .load_operations( operations ) ;
+
 	//set new statusbartext
 	statusbar .pop() ;
 	if ( operations .size() != 1 )
@@ -655,10 +665,7 @@ void Win_GParted::Refresh_Visual()
 		statusbar .push( _( "1 operation pending" ) );
 		
 	if ( ! operations .size() ) 
-	{
-		allow_undo( false );
-		allow_apply( false );
-	}
+		allow_undo_clear_apply( false ) ;
 			
 	//count primary's and check for extended
 	index_extended = -1 ;
@@ -729,11 +736,13 @@ void Win_GParted::set_valid_operations()
 {
 	allow_new( false ); allow_delete( false ); allow_resize( false ); allow_copy( false );
 	allow_paste( false ); allow_format( false ); allow_toggle_swap_mount_state( false ) ;
-	allow_manage_flags( false ) ; allow_info( false ) ;
+	allow_manage_flags( false ) ; allow_check( false ) ; allow_info( false ) ;
 	
-       	dynamic_cast<Gtk::Label*>(menu_partition .items()[ 10 ] .get_child() ) ->set_label( _("unmount") ) ;
-	menu_partition .items()[ 10 ] .show() ;
-	menu_partition .items()[ 11 ] .hide() ;	
+       	dynamic_cast<Gtk::Label*>( menu_partition .items()[ MENU_TOGGLE_MOUNT_SWAP ] .get_child() )
+		->set_label( _("_Unmount") ) ;
+
+	menu_partition .items()[ MENU_TOGGLE_MOUNT_SWAP ] .show() ;
+	menu_partition .items()[ MENU_MOUNT ] .hide() ;	
 
 	//no partition selected...	
 	if ( ! selected_partition .get_paths() .size() )
@@ -754,14 +763,14 @@ void Win_GParted::set_valid_operations()
 
 		if ( selected_partition .busy )
 		{
-			dynamic_cast<Gtk::Label*>(menu_partition .items()[ 10 ] .get_child() ) 
-				->set_label( _("swapoff") ) ;
+			dynamic_cast<Gtk::Label*>(menu_partition .items()[ MENU_TOGGLE_MOUNT_SWAP ] .get_child() ) 
+				->set_label( _("_Swapoff") ) ;
 
 			return ;
 		}
 		else
-       			dynamic_cast<Gtk::Label*>(menu_partition .items()[ 10 ] .get_child() ) 
-				->set_label( _("swapon") ) ;
+       			dynamic_cast<Gtk::Label*>(menu_partition .items()[ MENU_TOGGLE_MOUNT_SWAP ] .get_child() ) 
+				->set_label( _("_Swapon") ) ;
 	}
 
 	//only unmount is allowed (if ! extended)
@@ -828,7 +837,7 @@ void Win_GParted::set_valid_operations()
 		{
 			allow_toggle_swap_mount_state( true ) ;
 			
-			menu = menu_partition .items()[ 11 ] .get_submenu() ;
+			menu = menu_partition .items()[ MENU_MOUNT ] .get_submenu() ;
 			menu ->items() .clear() ;
 			for ( unsigned int t = 0 ; t < selected_partition .get_mountpoints() .size() ; t++ )
 			{
@@ -840,8 +849,8 @@ void Win_GParted::set_valid_operations()
 				dynamic_cast<Gtk::Label*>( menu ->items() .back() .get_child() ) ->set_use_underline( false ) ;
 			}
 			
-			menu_partition .items()[ 10 ] .hide() ;
-			menu_partition .items()[ 11 ] .show() ;	
+			menu_partition .items()[ MENU_TOGGLE_MOUNT_SWAP ] .hide() ;
+			menu_partition .items()[ MENU_MOUNT ] .show() ;	
 		}
 
 		//see if there is an copied partition and if it passes all tests
@@ -850,6 +859,10 @@ void Win_GParted::set_valid_operations()
 		     selected_partition .status == GParted::STAT_REAL &&
 		     copied_partition != selected_partition )
 		     allow_paste( true ) ;
+
+		//see if we can somehow check/repair this filesystem....
+		if ( fs .check && selected_partition .status == GParted::STAT_REAL )
+			allow_check( true ) ;
 	}
 }
 
@@ -896,6 +909,7 @@ void Win_GParted::close_operationslist()
 void Win_GParted::clear_operationslist() 
 {
 	remove_operation( -1, true ) ;
+	close_operationslist() ;
 
 	Refresh_Visual() ;
 }
@@ -938,10 +952,10 @@ void Win_GParted::on_show()
 
 	menu_gparted_refresh_devices() ;
 }
-
+	
 void Win_GParted::thread_refresh_devices() 
 {
-	gparted_core .get_devices( devices ) ;
+	gparted_core .set_devices( devices ) ;
 	pulse = false ;
 }
 
@@ -1012,7 +1026,7 @@ void Win_GParted::menu_gparted_refresh_devices()
 		treeview_detail .clear() ;
 		
 		//hmzz, this is really paranoid, but i think it's the right thing to do ;)
-		liststore_operations ->clear() ;
+		hbox_operations .clear() ;
 		close_operationslist() ;
 		remove_operation( -1, true ) ;
 		
@@ -1049,9 +1063,9 @@ void Win_GParted::menu_gparted_features()
 		dialog .load_filesystems( gparted_core .get_filesystems() ) ;
 
 		//recreate format menu...
-		menu_partition .items()[ 8 ] .remove_submenu() ;
-		menu_partition .items()[ 8 ] .set_submenu( * create_format_menu() ) ;
-		menu_partition .show_all_children() ;
+		menu_partition .items()[ MENU_FORMAT ] .remove_submenu() ;
+		menu_partition .items()[ MENU_FORMAT ] .set_submenu( * create_format_menu() ) ;
+		menu_partition .items()[ MENU_FORMAT ] .get_submenu() ->show_all_children() ;
 	}
 }
 
@@ -1168,6 +1182,8 @@ void Win_GParted::on_partition_popup_menu( unsigned int button, unsigned int tim
 
 bool Win_GParted::max_amount_prim_reached() 
 {
+	//FIXME: this is the only place where primary_count is used... instead of counting the primaries on each
+	//refresh, we could just count them here.
 	//Display error if user tries to create more primary partitions than the partition table can hold. 
 	if ( ! selected_partition .inside_extended && primary_count >= devices[ current_device ] .max_prims )
 	{
@@ -1235,14 +1251,27 @@ void Win_GParted::activate_resize()
 									     selected_partition .sector_start,
 									     selected_partition .sector_end,
 									     selected_partition .inside_extended ) ;
-					Add_Operation( GParted::CREATE, dialog .Get_New_Partition() );
-					
+
+					Operation * operation = new OperationCreate( devices[ current_device ],
+							 			     selected_partition,
+							 			     dialog .Get_New_Partition() ) ;
+					operation ->icon = render_icon( Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU );
+
+					Add_Operation( operation ) ;
+
 					break;
 				}
 			}
 		}
 		else//normal move/resize on existing partition
-			Add_Operation( GParted::RESIZE_MOVE, dialog .Get_New_Partition() );
+		{
+			Operation * operation = new OperationResizeMove( devices[ current_device ],
+					 			         selected_partition,
+							     		 dialog .Get_New_Partition() );
+			operation ->icon = render_icon( Gtk::Stock::GOTO_LAST, Gtk::ICON_SIZE_MENU );
+
+			Add_Operation( operation ) ;
+		}
 	}
 }
 
@@ -1253,15 +1282,23 @@ void Win_GParted::activate_copy()
 
 void Win_GParted::activate_paste()
 {
+	//if max_prims == -1 the current device has an unrecognised disklabel (see also GParted_Core::get_devices)
+	if ( devices [ current_device ] .max_prims == -1 )
+	{
+		//FIXME: actually we should proceed with pasting after the new disklabel is set.
+		//(the same goes when creating a new partition on a disk without disklabel)
+		activate_disklabel() ;
+		return ;
+	}
+
 	if ( selected_partition .type == GParted::TYPE_UNALLOCATED )
 	{
-
 		if ( ! max_amount_prim_reached() )
 		{
 			Dialog_Partition_Copy dialog( gparted_core .get_fs( copied_partition .filesystem ),
 						      devices[ current_device ] .cylsize ) ;
-			//we don't need the errors/mountpoints of the source partition.
-			copied_partition .error .clear() ;
+			//we don't need the messages/mountpoints of the source partition.
+			copied_partition .messages .clear() ;
 			copied_partition .clear_mountpoints() ;
 			dialog .Set_Data( selected_partition, copied_partition ) ;
 			dialog .set_transient_for( *this );
@@ -1269,7 +1306,14 @@ void Win_GParted::activate_paste()
 			if ( dialog .run() == Gtk::RESPONSE_OK )
 			{
 				dialog .hide() ;
-				Add_Operation( GParted::COPY, dialog .Get_New_Partition() );		
+
+				Operation * operation = new OperationCopy( devices[ current_device ],
+					 	       			   selected_partition,
+						       			   dialog .Get_New_Partition(),
+						       			   copied_partition ) ;
+				operation ->icon = render_icon( Gtk::Stock::COPY, Gtk::ICON_SIZE_MENU );
+
+				Add_Operation( operation ) ;
 			}
 		}
 	}
@@ -1279,10 +1323,15 @@ void Win_GParted::activate_paste()
 		partition_new .filesystem = copied_partition .filesystem ;
 		partition_new .color = copied_partition .color ;
 		partition_new .set_used( copied_partition .sectors_used ) ;
-		partition_new .error .clear() ;
-		partition_new .status = GParted::STAT_COPY ;
+		partition_new .messages .clear() ;
+ 
+		Operation * operation = new OperationCopy( devices[ current_device ],
+					 	       	   selected_partition,
+							   partition_new,
+						       	   copied_partition ) ;
+		operation ->icon = render_icon( Gtk::Stock::COPY, Gtk::ICON_SIZE_MENU );
 
-		Add_Operation( GParted::COPY, partition_new ) ;
+		Add_Operation( operation ) ;
 	}
 }
 
@@ -1310,7 +1359,12 @@ void Win_GParted::activate_new()
 			dialog .hide() ;
 			
 			new_count++ ;
-			Add_Operation( GParted::CREATE, dialog .Get_New_Partition() );
+			Operation *operation = new OperationCreate( devices[ current_device ],
+							 	    selected_partition,
+							 	    dialog .Get_New_Partition() ) ;
+			operation ->icon = render_icon( Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU );
+
+			Add_Operation( operation );
 		}
 	}
 }
@@ -1398,8 +1452,13 @@ void Win_GParted::activate_delete()
 		if ( ! operations .size() )
 			close_operationslist() ;
 	}
-	else //deletion of a real partition...(now selected_partition is just a dummy)
-		Add_Operation( GParted::DELETE, selected_partition );
+	else //deletion of a real partition...
+	{
+		Operation * operation = new OperationDelete( devices[ current_device ], selected_partition ) ;
+		operation ->icon = render_icon( Gtk::Stock::DELETE, Gtk::ICON_SIZE_MENU ) ;
+
+		Add_Operation( operation ) ;
+	}
 }
 
 void Win_GParted::activate_info()
@@ -1465,18 +1524,30 @@ void Win_GParted::activate_format( GParted::FILESYSTEM new_fs )
 			{
 				remove_operation( t ) ;
 				
-				//And add the new partition to the end of the operations list
+				//And insert the new partition at the old position in the operations list
 				//(NOTE: in this case we set status to STAT_NEW)
 				part_temp .status = STAT_NEW ;
 				
-				Add_Operation( GParted::CREATE, part_temp, t );
+				Operation * operation = new OperationCreate( devices[ current_device ],
+									     selected_partition,
+								 	     part_temp ) ;
+				operation ->icon = render_icon( Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU );
+
+				Add_Operation( operation, t ) ;
 					
 				break;
 			}
 		}
 	}
 	else//normal formatting of an existing partition
-		Add_Operation( GParted::FORMAT, part_temp ) ;
+	{
+		Operation *operation = new OperationFormat( devices[ current_device ],
+				 			    selected_partition,
+						 	    part_temp );
+		operation ->icon = render_icon( Gtk::Stock::CONVERT, Gtk::ICON_SIZE_MENU );
+
+		Add_Operation( operation ) ;
+	}
 }
 
 void Win_GParted::thread_unmount_partition( bool * succes, Glib::ustring * error ) 
@@ -1659,7 +1730,7 @@ void Win_GParted::activate_disklabel()
 		m_dialog .add_button( _("Create"), Gtk::RESPONSE_OK ) ;
 		
 		if ( m_dialog .run() == Gtk::RESPONSE_OK && 
-		     ! gparted_core .Set_Disklabel( devices[ current_device ] .get_path(), dialog .Get_Disklabel() ) )
+		     ! gparted_core .set_disklabel( devices[ current_device ] .get_path(), dialog .Get_Disklabel() ) )
 		{
 			Gtk::MessageDialog dialog( *this,
 						   _("Error while setting new disklabel"),
@@ -1698,11 +1769,20 @@ void Win_GParted::activate_manage_flags()
 	if ( dialog .any_change )
 		menu_gparted_refresh_devices() ;
 }
+	
+void Win_GParted::activate_check() 
+{
+	Operation *operation = new OperationCheck( devices[ current_device ], selected_partition ) ;
+
+	operation ->icon = render_icon( Gtk::Stock::EXECUTE, Gtk::ICON_SIZE_MENU );
+
+	Add_Operation( operation ) ;
+}
 
 void Win_GParted::activate_undo()
 {
-	//when undoing an creation it's safe to decrease the newcount by one
-	if ( operations .back() ->type == GParted::CREATE )
+	//when undoing a creation it's safe to decrease the newcount by one
+	if ( operations .back() ->type == OPERATION_CREATE )
 		new_count-- ;
 
 	remove_operation() ;		
@@ -1757,6 +1837,8 @@ void Win_GParted::activate_apply()
 		dialog_progress .set_transient_for( *this ) ;
 		dialog_progress .signal_apply_operation .connect(
 			sigc::mem_fun(gparted_core, &GParted_Core::apply_operation_to_disk) ) ;
+		dialog_progress .signal_get_libparted_version .connect(
+			sigc::mem_fun(gparted_core, &GParted_Core::get_libparted_version) ) ;
  
 		int response ;
 		do
@@ -1769,7 +1851,7 @@ void Win_GParted::activate_apply()
 		
 		//wipe operations...
 		remove_operation( -1, true ) ;
-		liststore_operations ->clear() ;
+		hbox_operations .clear() ;
 		close_operationslist() ;
 							
 		//reset new_count to 1
